package com.tzld.piaoquan.recommend.model.produce.xgboost;

import com.aliyun.odps.utils.StringUtils;
import lombok.extern.slf4j.Slf4j;
import ml.dmlc.xgboost4j.scala.spark.XGBoostClassificationModel;
import ml.dmlc.xgboost4j.scala.spark.XGBoostClassifier;
import org.apache.commons.lang.math.NumberUtils;
import org.apache.spark.api.java.JavaRDD;
import org.apache.spark.api.java.JavaSparkContext;
import org.apache.spark.sql.Dataset;
import org.apache.spark.sql.Row;
import org.apache.spark.sql.RowFactory;
import org.apache.spark.sql.SparkSession;

/**
 * @author dyp
 */
@Slf4j
public class XGBoostTrain {
    public static void main(String[] args) {
        try {
            SparkSession spark = SparkSession.builder()
                    .appName("XGBoostTrain")
                    .master("local")
                    .getOrCreate();

            JavaSparkContext jsc = new JavaSparkContext(spark.sparkContext());
            String file = "hdfs://dw/recommend/model/33_ad_train_data_v3_checkfeature/20240703/part-00099.gz";
            JavaRDD<String> rdd = jsc.textFile(file);

            // 将 RDD[LabeledPoint] 转换为 JavaRDD<Row>
            JavaRDD<Row> rowRDD = rdd.map(s -> {
                String[] line = StringUtils.split("\t");
                String label = line[0];
                double[] values = new double[line.length - 1];
                for (int i = 1; i < line.length; i++) {
                    String[] fv = StringUtils.split(":");
                    values[i - 1] = NumberUtils.toDouble(fv[1], 0.0);
                }
                return RowFactory.create(label, values);
            });

            // 将 JavaRDD<Row> 转换为 Dataset<Row>
            Dataset<Row> dataset = spark.createDataFrame(rowRDD, Row.class);

            // 如果需要，可以添加列名
            dataset = dataset.toDF("label", "features");

            // 划分训练集和测试集
            Dataset<Row>[] splits = dataset.randomSplit(new double[]{0.7, 0.3});
            Dataset<Row> trainData = splits[0];
            Dataset<Row> testData = splits[1];

            // 参数


            // 创建 XGBoostClassifier 对象
            XGBoostClassifier xgbClassifier = new XGBoostClassifier()
                    .setEta(0.1f)
                    .setFeaturesCol("features")
                    .setLabelCol("label")
                    .setMaxDepth(5)
                    .setObjective("binary:logistic")
                    .setNthread(4)
                    .setNumRound(10)
                    .setNumWorkers(2);


            // 训练模型
            XGBoostClassificationModel model = xgbClassifier.fit(trainData);

            // 显示预测结果
            Dataset<Row> predictions = model.transform(testData);
            predictions.select("label", "prediction").show();

        } catch (Exception e) {
            log.error("", e);
        }
    }
}
